UNIT LRLSprites;

INTERFACE

USES DOSFileAccess,LRLRoutines;

{
	all coordinates in standard style:
	0          +
  0 +---------->  x
	|
	|
	|
  + v

	y
}

TYPE
	TLRLImage = RECORD
		Data: POINTER; { standard 256-colour image data }
		Size: WORD; { size of image (for destruction) }
	END;

	PLRLSprite = ^TLRLSprite;
	TLRLSprite = RECORD
		Image: ARRAY[1..10] OF TLRLImage; { moving image }
		ImageCount: BYTE; { how many images there }
	END;
TYPE
	TButton = RECORD
		Lit: BOOLEAN;
		DarkIcon: POINTER;
		LightIcon: POINTER;
		DarkIconSize: WORD;
		LightIconSize: WORD;
		x1,y1: INTEGER;
		x2,y2: INTEGER;
		Command: WORD;
	END;
VAR
	ImageFile: TDFAFileHandle;
	LRLEnvironment: ARRAY[1..20] OF TLRLSprite;
	LRLFigure: ARRAY[1..4,1..9] OF TLRLSprite;
	LRLDecoration: ARRAY[1..1] OF TLRLSprite;
	LRLPalette: POINTER;
	LRLScreen: POINTER;
	LRLMenuPalette: POINTER;
	LRLLogo: POINTER;
	LRLFont: POINTER;
	LRLFontBuffer: POINTER;
	LRLMousePointer: POINTER;
	LRLImagesFilePosition: LONGINT;
	LRLEditorButton: ARRAY[1..6] OF TButton;

PROCEDURE ImagesInitialize;
PROCEDURE ImagesDeinitialize;

IMPLEMENTATION
CONST
	ImageFileName = 'LRL.IMG';
	ImageFileHeader: STRING[24] = 'Lode Runner Live Images'#26;

PROCEDURE LoadImages;
VAR
	InBuffer: POINTER;
	i,j,k,l,x,y: WORD;
	a,b,c: BYTE;
BEGIN
	GETMEM(InBuffer,$FFF0);
	DFAFileOpen(ImageFile,ImageFileName,DFAAccessRead);
	IF DFALastResult(ImageFile)<>0 THEN BEGIN
		WRITELN('Error: Cannot open file with images!');
		WRITELN('訡:   䠩 ⨭!');
		FHALT;
	END;
	DFAFileRead(ImageFile,InBuffer^,24,k);
	IF (DFALastResult(ImageFile)<>0) OR (NOT DataIdentical(InBuffer^,ImageFileHeader[1],24,0,0)) THEN BEGIN
		WRITELN('Error: Invalid or damaged file with images!');
		WRITELN('訡:   ० 䠩 ⨭!');
		FHALT;
	END;
	GETMEM(LRLPalette,768); { load palette }
	DFAFileRead(ImageFile,LRLPalette^,768,k);
	FOR i:=1 TO 9 DO BEGIN { figures loading loop }
		DFAFileRead(ImageFile,a,1,k);
		LRLFigure[1,i].ImageCount:=a;
		FOR j:=1 TO a DO BEGIN
			GETMEM(LRLFigure[1,i].Image[j].Data,104);
			DFAFileRead(ImageFile,LRLFigure[1,i].Image[j].Data^,104,k);
			x:=DataWordGet(LRLFigure[1,i].Image[j].Data^,0);
			y:=DataWordGet(LRLFigure[1,i].Image[j].Data^,2);
			LRLFigure[1,i].Image[j].Size:=x*y+4;
			FOR l:=2 TO 4 DO BEGIN
				LRLFigure[l,i].Image[j].Size:=LRLFigure[1,i].Image[j].Size;
				LRLFigure[l,i].ImageCount:=a;
				GETMEM(LRLFigure[l,i].Image[j].Data,LRLFigure[l,i].Image[j].Size);
				DataMove(LRLFigure[1,i].Image[j].Data^,LRLFigure[l,i].Image[j].Data^,LRLFigure[l,i].Image[j].Size,0,0);
				DataAdd(LRLFigure[l,i].Image[j].Data^,LRLFigure[l,i].Image[j].Size,(l-1) SHL 5,4);
			END;
		END;
	END;
	FOR i:=1 TO 1 DO BEGIN { decoration loading loop }
		DFAFileRead(ImageFile,a,1,k);
		LRLDecoration[i].ImageCount:=a;
		FOR j:=1 TO a DO BEGIN
			GETMEM(LRLDecoration[i].Image[j].Data,104);
			DFAFileRead(ImageFile,LRLDecoration[i].Image[j].Data^,104,k);
			x:=DataWordGet(LRLDecoration[i].Image[j].Data^,0);
			y:=DataWordGet(LRLDecoration[i].Image[j].Data^,2);
			LRLDecoration[i].Image[j].Size:=x*y+4;
		END;
	END;
	FOR i:=1 TO 9 DO BEGIN { environment loading loop }
		DFAFileRead(ImageFile,a,1,k);
		LRLEnvironment[i].ImageCount:=a;
		FOR j:=1 TO a DO BEGIN
			GETMEM(LRLEnvironment[i].Image[j].Data,104);
			DFAFileRead(ImageFile,LRLEnvironment[i].Image[j].Data^,104,k);
			x:=DataWordGet(LRLEnvironment[i].Image[j].Data^,0);
			y:=DataWordGet(LRLEnvironment[i].Image[j].Data^,2);
			LRLEnvironment[i].Image[j].Size:=x*y+4;
		END;
	END;
	y:=181;
	x:=212;
	FOR i:=1 TO 6 DO BEGIN
		IF i=4 THEN BEGIN
			y:=191;
			x:=212;
		END;
		LRLEditorButton[i].x1:=x;
		LRLEditorButton[i].y1:=y;
		LRLEditorButton[i].x2:=x+32;
		LRLEditorButton[i].y2:=y+9;
		LRLEditorButton[i].Lit:=FALSE;
		LRLEditorButton[i].Command:=i;
		LRLEditorButton[i].DarkIconSize:=292;
		LRLEditorButton[i].LightIconSize:=292;
		GETMEM(LRLEditorButton[i].DarkIcon,LRLEditorButton[i].DarkIconSize);
		GETMEM(LRLEditorButton[i].LightIcon,LRLEditorButton[i].LightIconSize);
		DFAFileRead(ImageFile,LRLEditorButton[i].LightIcon^,292,l);
		DFAFileRead(ImageFile,LRLEditorButton[i].DarkIcon^,292,l);
		x:=x+33;
	END;
	GETMEM(LRLFont,20455); { load font }
	DFAFileRead(ImageFile,LRLFont^,20455,k);
	GETMEM(LRLMousePointer,174); { load pointer }
	DFAFileRead(ImageFile,LRLMousePointer^,174,k);
	GETMEM(LRLMenuPalette,768); { load palette }
	DFAFileRead(ImageFile,LRLMenuPalette^,768,k);
	GETMEM(LRLLogo,12524); { load logo }
	DFAFileRead(ImageFile,LRLLogo^,12524,k);
	LRLImagesFilePosition:=DFAFilePositionGet(ImageFile);
	FREEMEM(InBuffer,$FFF0);
END;

PROCEDURE ImagesInitialize;
BEGIN
	LoadImages;
	GETMEM(LRLScreen,64000);
	GETMEM(LRLFontBuffer,32000);
END;

PROCEDURE ImagesDeinitialize;
VAR
	i,j,l: INTEGER;
BEGIN
	FREEMEM(LRLPalette,768);
	FOR i:=1 TO 9 DO BEGIN
		FOR j:=1 TO LRLFigure[1,i].ImageCount DO BEGIN
			FREEMEM(LRLFigure[1,i].Image[j].Data,104);
			FOR l:=2 TO 4 DO BEGIN
				FREEMEM(LRLFigure[l,i].Image[j].Data,LRLFigure[l,i].Image[j].Size);
			END;
		END;
	END;
	FOR i:=1 TO 1 DO BEGIN
		FOR j:=1 TO LRLDecoration[i].ImageCount DO BEGIN
			FREEMEM(LRLDecoration[i].Image[j].Data,104);
		END;
	END;
	FOR i:=1 TO 9 DO BEGIN
		FOR j:=1 TO LRLEnvironment[i].ImageCount DO BEGIN
			FREEMEM(LRLEnvironment[i].Image[j].Data,104);
		END;
	END;
	FOR i:=1 TO 6 DO BEGIN
		FREEMEM(LRLEditorButton[i].DarkIcon,LRLEditorButton[i].DarkIconSize);
		FREEMEM(LRLEditorButton[i].LightIcon,LRLEditorButton[i].LightIconSize);
	END;
	FREEMEM(LRLFont,20455);
	FREEMEM(LRLMousePointer,174);
	FREEMEM(LRLMenuPalette,768);
	FREEMEM(LRLLogo,12524);
	FREEMEM(LRLScreen,64000);
	FREEMEM(LRLFontBuffer,32000);
	DFAFileClose(ImageFile);
END;
END.

